
"""Version information for agensight package."""

__version__ = "0.1.0"
__version_info__ = tuple(int(i) for i in __version__.split('.') if i.isdigit())

# Build information (will be updated by CI/CD)
__build__ = None
__commit__ = None