(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[293],{2432:(e,t,r)=>{"use strict";r.d(t,{A:()=>l});let l=(0,r(9946).A)("panel-left",[["rect",{width:"18",height:"18",x:"3",y:"3",rx:"2",key:"afitv7"}],["path",{d:"M9 3v18",key:"fh3hqa"}]])},4168:(e,t,r)=>{"use strict";r.d(t,{A:()=>l});var l=(0,r(6467).A)("outline","moon","IconMoon",[["path",{d:"M12 3c.132 0 .263 0 .393 0a7.5 7.5 0 0 0 7.92 12.446a9 9 0 1 1 -8.313 -12.454z",key:"svg-0"}]])},4615:(e,t,r)=>{"use strict";r.d(t,{Y:()=>c});var l=r(5155);r(2115);var s=r(8185),a=r(4168),i=r(285),n=r(6102),d=r(2954);function c(e){let{darkMode:t,toggleDarkMode:r,children:c}=e;return(0,l.jsx)("header",{className:"sticky top-0 z-50 border-b border-border/40 bg-card/50 backdrop-blur-sm w-full",children:(0,l.jsxs)("div",{className:"pr-6 pl-3 flex h-14 justify-between w-full items-center",children:[(0,l.jsxs)("div",{className:"flex items-center gap-3",children:[(0,l.jsx)(d.x2,{}),(0,l.jsx)("div",{className:"flex items-center gap-4",children:c})]}),(0,l.jsx)("div",{className:"flex items-center",children:(0,l.jsx)(n.Bc,{children:(0,l.jsxs)(n.m_,{children:[(0,l.jsx)(n.k$,{asChild:!0,children:(0,l.jsx)(i.$,{variant:"outline",size:"icon",className:"rounded-full",onClick:r,children:t?(0,l.jsx)(s.A,{className:"h-4 w-4"}):(0,l.jsx)(a.A,{className:"h-4 w-4"})})}),(0,l.jsx)(n.ZI,{children:(0,l.jsx)("p",{children:"Toggle theme"})})]})})})]})})}},5909:(e,t,r)=>{Promise.resolve().then(r.bind(r,7776))},6467:(e,t,r)=>{"use strict";r.d(t,{A:()=>a});var l=r(2115),s={outline:{xmlns:"http://www.w3.org/2000/svg",width:24,height:24,viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:2,strokeLinecap:"round",strokeLinejoin:"round"},filled:{xmlns:"http://www.w3.org/2000/svg",width:24,height:24,viewBox:"0 0 24 24",fill:"currentColor",stroke:"none"}};let a=(e,t,r,a)=>{let i=(0,l.forwardRef)((r,i)=>{let{color:n="currentColor",size:d=24,stroke:c=2,title:o,className:h,children:u,...m}=r;return(0,l.createElement)("svg",{ref:i,...s[e],width:d,height:d,className:["tabler-icon","tabler-icon-".concat(t),h].join(" "),..."filled"===e?{fill:n}:{strokeWidth:c,stroke:n},...m},[o&&(0,l.createElement)("title",{key:"svg-title"},o),...a.map(e=>{let[t,r]=e;return(0,l.createElement)(t,r)}),...Array.isArray(u)?u:[u]])});return i.displayName="".concat(r),i}},7280:(e,t,r)=>{"use strict";r.d(t,{D:()=>n,N:()=>i});var l=r(5155),s=r(2115);let a=(0,s.createContext)({theme:"system",setTheme:()=>null,darkMode:!1,toggleDarkMode:()=>null});function i(e){let{children:t,defaultTheme:r="system",enableSystem:i=!0,attribute:n="data-theme"}=e,[d,c]=(0,s.useState)(r),[o,h]=(0,s.useState)("dark"===r);return(0,s.useEffect)(()=>{let e=window.document.documentElement;if(e.classList.remove("light","dark"),"system"===d&&i){let t=window.matchMedia("(prefers-color-scheme: dark)").matches?"dark":"light";e.classList.add(t),h("dark"===t)}else e.classList.add(d),h("dark"===d)},[d,i]),(0,l.jsx)(a.Provider,{value:{theme:d,setTheme:c,darkMode:o,toggleDarkMode:()=>{c(o?"light":"dark")}},children:t})}let n=()=>{let e=(0,s.useContext)(a);if(void 0===e)throw Error("useTheme must be used within a ThemeProvider");return e}},7776:(e,t,r)=>{"use strict";r.r(t),r.d(t,{default:()=>n});var l=r(5155),s=r(7280),a=r(2954),i=r(4615);function n(e){let{children:t}=e,{open:r}=(0,a.cL)(),{darkMode:n,toggleDarkMode:d}=(0,s.D)();return(0,l.jsx)("div",{className:"h-full bg-background w-full",children:(0,l.jsx)("div",{className:"flex h-full w-full",children:(0,l.jsxs)("main",{className:"transition-all ".concat(r?"ml-0":"ml-18"," w-full duration-200"),children:[(0,l.jsx)(i.Y,{darkMode:n,toggleDarkMode:d}),t]})})})}},8185:(e,t,r)=>{"use strict";r.d(t,{A:()=>l});var l=(0,r(6467).A)("outline","sun","IconSun",[["path",{d:"M12 12m-4 0a4 4 0 1 0 8 0a4 4 0 1 0 -8 0",key:"svg-0"}],["path",{d:"M3 12h1m8 -9v1m8 8h1m-9 8v1m-6.4 -15.4l.7 .7m12.1 -.7l-.7 .7m0 11.4l.7 .7m-12.1 -.7l-.7 .7",key:"svg-1"}]])}},e=>{var t=t=>e(e.s=t);e.O(0,[126,423,573,441,684,358],()=>t(5909)),_N_E=e.O()}]);